@@ -23,14 +23,18 @@ class OracleTools:
 
     @staticmethod
     def desc(ora_conn, username, table):
-        """ 类似于sqlplus中的desc命令 """
+        """ 输出一个表的，列名、字段类型、主键、索引（索引所在的列而非索引名）和默认值 """
         username = username.upper()
         table = table.upper()
+        
+        # 列
         sql = '''
             select column_name, 
                 case 
                     when data_type = 'VARCHAR2' or data_type = 'CHAR' or data_type = 'VARCHAR' then 
                         data_type || '(' || data_length || ')'
+                    when data_type = 'NVARCHAR' or data_type = 'NVARCHAR2' then 
+                        data_type || '(' || char_length || ')'
                     when data_type = 'NUMBER' and data_precision > 0 and data_scale > 0 then 
                         data_type || '(' || data_precision || ', ' || data_scale || ')'
                     when data_type = 'NUMBER' and data_precision > 0 and data_scale = 0 then 
@@ -53,11 +57,100 @@ class OracleTools:
             spacesnum = lenNum - len(tablename) + 1
             colstr = tablename + ' ' * (spacesnum) + typestr
             tab = tab + colstr + '\n'
+            
+        # 主键
+        sql = '''
+            select constraint_name
+            , max(cols)
+            from (
+                select t.constraint_name
+                , to_char(
+                    wm_concat(t2.column_name) 
+                    over(partition by t.constraint_name order by t2.position)
+                ) cols
+                from dba_constraints t, dba_cons_columns t2
+                where t.owner = ?
+                and t.table_name = ?
+                and t.owner = t2.owner
+                and t.constraint_name = t2.constraint_name
+                and t.table_name = t2.table_name
+                and t.constraint_type = 'P'
+            ) group by constraint_name
+        '''
+        rss = ora_conn.run(sql, (username, table)).get_rows()
+        for rs in rss:
+            mess = 'pk: (' + rs[1] + ')'
+            tab = tab + mess + '\n'
+        
+        # 索引，包括普通索引和普通函数索引
+        sql = '''
+            select t.index_name
+            , t.index_type
+            , t2.column_name
+            , t3.column_expression
+            , t2.descend
+            , t2.column_position
+            from dba_indexes t, dba_ind_columns t2, dba_ind_expressions t3
+            where t.table_owner = ? 
+            and t.table_name = ? 
+            and t.owner = t2.index_owner
+            and t.table_owner = t2.table_owner
+            and t.index_name = t2.index_name
+            and t.table_name = t2.table_name
+            and t2.index_owner = t3.index_owner(+)
+            and t2.table_owner = t3.table_owner(+)
+            and t2.index_name = t3.index_name(+)
+            and t2.table_name = t3.table_name(+)
+            and t2.column_position = t3.column_position(+)
+            and t.uniqueness = 'NONUNIQUE'
+            and t.index_type in('FUNCTION-BASED NORMAL', 'NORMAL')
+            order by t2.column_position
+        '''
+        rows = ora_conn.run(sql, (username, table)).get_rows()
+        if len(rows) > 0:
+            col = ora_conn.run(sql, (username, table)).get_cols()
+            mdb = MemoryDB()
+            mdb.set('t_idx', col, rows)
+            # 降序索引也体现为函数索引，列名会用"col_name"，而字符串用'str'
+            sql = '''
+                select distinct index_name
+                , group_concat(
+                    replace(case when column_name like 'SYS_%$' then column_expression else column_name end, '"', '')
+                    || ' ' || descend
+                ) over(
+                    partition by index_name 
+                    order by column_position 
+                    rows between unbounded preceding and unbounded following
+                ) index_str
+                from t_idx 
+            '''
+            rss = mdb.db_conn.run(sql).get_rows()
+            mdb.close()
+            for rs in rss:
+                mess = 'index: (' + rs[1] + ')'
+                tab = tab + mess + '\n'
+            
+        # 默认值
+        sql = '''
+            select column_name, data_default
+            from dba_tab_columns
+            where owner = ? 
+            and table_name = ? 
+            and column_name not like '%$%'
+            and data_default is not null
+            order by column_id
+        '''
+        rss = ora_conn.run(sql, (username, table)).get_rows()
+        for rs in rss:
+            mess = 'data_default: ' + rs[0] + ' ' + rs[1].replace('  /* GOLDENGATE_DDL_REPLICATION */', '').strip().strip('\n')
+            tab = tab + mess + '\n'
+            
         return tab
     
     @staticmethod
     def get_ddl(dba_conn, username, table):
-        """ 需要dba权限，得到目标user.table的单元性的建表语句，包括约束，索引和注释等 """
+        """ 得到目标user.table的单元性的建表语句，包括按顺序的列名、列类型、默认值、非空约束，普通索引、函数索引和注释 """
+        """ 不包括外键、分区信息、存储信息和特殊索引类型 """
         username = username.upper()
         table = table.upper()
         rssqls = []
@@ -67,8 +160,10 @@ class OracleTools:
         sql = '''
             select column_name, 
                 case 
-                    when data_type = 'VARCHAR2' or data_type = 'CHAR' or data_type = 'VARCHAR' or data_type = 'NVARCHAR2' then 
+                    when data_type = 'VARCHAR2' or data_type = 'CHAR' or data_type = 'VARCHAR' then 
                         data_type || '(' || data_length || ')'
+                    when data_type = 'NVARCHAR' or data_type = 'NVARCHAR2' then 
+                        data_type || '(' || char_length || ')'
                     when data_type = 'NUMBER' and data_precision > 0 and data_scale > 0 then 
                         data_type || '(' || data_precision || ', ' || data_scale || ')'
                     when data_type = 'NUMBER' and data_precision > 0 and data_scale = 0 then 
@@ -84,14 +179,22 @@ class OracleTools:
         rssqls.append('alter table ' + username + '.' + table + ' drop column test_col')
         # 建主键
         sql = '''
-            select t.constraint_name, to_char(wm_concat(t2.column_name)) cols
-            from dba_constraints t, dba_cons_columns t2
-            where t.owner = ?
-            and t.table_name = ?
-            and t.constraint_name = t2.constraint_name
-            and t.table_name = t2.table_name
-            and t.constraint_type = 'P'
-            group by t.constraint_name
+            select constraint_name
+            , max(cols)
+            from (
+                select t.constraint_name
+                , to_char(
+                    wm_concat(t2.column_name) 
+                    over(partition by t.constraint_name order by t2.position)
+                ) cols
+                from dba_constraints t, dba_cons_columns t2
+                where t.owner = ?
+                and t.table_name = ?
+                and t.owner = t2.owner
+                and t.constraint_name = t2.constraint_name
+                and t.table_name = t2.table_name
+                and t.constraint_type = 'P'
+            ) group by constraint_name
         '''
         for r in dba_conn.run(sql, (username, table)).get_rows():
             rssqls.append('alter table ' + username + '.' + table + ' add constraint ' + r[0] + ' primary key(' + r[1] + ')')
@@ -101,6 +204,7 @@ class OracleTools:
             from dba_constraints t, dba_cons_columns t2
             where t.owner = ?
             and t.table_name = ?
+            and t.owner = t2.owner
             and t.constraint_name = t2.constraint_name
             and t.table_name = t2.table_name
             and t.constraint_type = 'C'
@@ -112,14 +216,22 @@ class OracleTools:
                 rssqls.append('alter table ' + username + '.' + table + ' modify ' + col + ' not null')
         # 建唯一约束
         sql = '''
-            select t.constraint_name, to_char(wm_concat(t2.column_name)) cols
-            from dba_constraints t, dba_cons_columns t2
-            where t.owner = ?
-            and t.table_name = ?
-            and t.constraint_name = t2.constraint_name
-            and t.table_name = t2.table_name
-            and t.constraint_type = 'U'
-            group by t.constraint_name
+            select constraint_name
+            , max(cols)
+            from (
+                select t.constraint_name
+                , to_char(
+                    wm_concat(t2.column_name)
+                    over(partition by t.constraint_name order by t2.position)
+                ) cols
+                from dba_constraints t, dba_cons_columns t2
+                where t.owner = ?
+                and t.table_name = ?
+                and t.owner = t2.owner
+                and t.constraint_name = t2.constraint_name
+                and t.table_name = t2.table_name
+                and t.constraint_type = 'U'
+            ) group by constraint_name
         '''
         for r in dba_conn.run(sql, (username, table)).get_rows():
             rssqls.append('alter table ' + username + '.' + table + ' add constraint ' + r[0] + ' unique(' + r[1] + ')')
@@ -131,43 +243,56 @@ class OracleTools:
             and table_name = ? 
             and column_name not like '%$%'
             and data_default is not null
+            order by column_id
         '''
         for r in dba_conn.run(sql, (username, table)).get_rows():
-            rssqls.append('alter table ' + username + '.' + table + ' modify ' + r[0] + ' default ' + r[1].strip())
-        # 建普通索引
+            rssqls.append('alter table ' + username + '.' + table + ' modify ' + r[0] + ' default ' + r[1].strip().strip('\n'))
+        # 建索引，包括普通索引和普通函数索引
         sql = '''
-            select t.index_name, to_char(wm_concat(t2.column_name)) cols
-            from dba_indexes t, dba_ind_columns t2
-            where t.owner = ? 
+            select t.index_name
+            , t.index_type
+            , t2.column_name
+            , t3.column_expression
+            , t2.descend
+            , t2.column_position
+            from dba_indexes t, dba_ind_columns t2, dba_ind_expressions t3
+            where t.table_owner = ? 
             and t.table_name = ? 
+            and t.owner = t2.index_owner
+            and t.table_owner = t2.table_owner
             and t.index_name = t2.index_name
-            and t.owner = t2.table_owner
+            and t.table_name = t2.table_name
+            and t2.index_owner = t3.index_owner(+)
+            and t2.table_owner = t3.table_owner(+)
+            and t2.index_name = t3.index_name(+)
+            and t2.table_name = t3.table_name(+)
+            and t2.column_position = t3.column_position(+)
             and t.uniqueness = 'NONUNIQUE'
-            and t.index_type = 'NORMAL'
-            group by t.index_name
+            and t.index_type in('FUNCTION-BASED NORMAL', 'NORMAL')
+            order by t2.column_position
         '''
-        for r in dba_conn.run(sql, (username, table)).get_rows():
-            rssqls.append('create index ' + username + '.' + r[0] + ' on ' + username + '.' + table + '(' + r[1] + ')')
-        # 建函数索引
-        sql = '''
-            select t.index_name, t3.column_expression
-            from dba_indexes t, dba_ind_expressions t3
-            where t.owner = ? 
-            and t.table_name = ? 
-            and t.index_name = t3.index_name
-            and t.table_name = t3.table_name
-            and t.owner = t3.table_owner
-            and t.uniqueness = 'NONUNIQUE'
-            and t.index_type = 'FUNCTION-BASED NORMAL'
-            order by t3.column_position
-        '''
-        col, rows = dba_conn.run(sql, (username, table))
-        mdb = MemoryDB()
-        mdb.set('t_idx', col, rows)
-        rs = mdb.ora_conn.run('select index_name, group_concat(column_expression) from t_idx group by index_name').get_rows()
-        mdb.close()
-        for r in rs:
-            rssqls.append('create index ' + username + '.' + r[0] + ' on ' + username + '.' + table + '(' + r[1] + ')')
+        rows = dba_conn.run(sql, (username, table)).get_rows()
+        if len(rows) > 0:
+            col = dba_conn.run(sql, (username, table)).get_cols()
+            mdb = MemoryDB()
+            mdb.set('t_idx', col, rows)
+            # 降序索引也体现为函数索引，列名会用"col_name"，而字符串用'str'
+            sql = '''
+                select distinct index_name
+                , group_concat(
+                    replace(case when column_name like 'SYS_%$' then column_expression else column_name end, '"', '')
+                    || ' ' || descend
+                ) over(
+                    partition by index_name 
+                    order by column_position 
+                    rows between unbounded preceding and unbounded following
+                ) index_str
+                from t_idx 
+            '''
+            rs = mdb.db_conn.run(sql).get_rows()
+            mdb.close()
+            for r in rs:
+                rssqls.append('create index ' + username + '.' + r[0] + ' on ' + username + '.' + table + '(' + r[1] + ')')
         # 建注释
         sql = '''
             select column_name, comments
@@ -183,7 +308,7 @@ class OracleTools:
         return rssqls
     
     @staticmethod
-    def get_dbms_ddl(dba_conn, username, table):
+    def get_ddl_by_dbms(dba_conn, username, table):
         """ 得到目标user.table的的建表语句，直接调用dbms_metadata.get_ddl得到结果 """
         username = username.upper()
         table = table.upper()
@@ -195,109 +320,27 @@ class OracleTools:
         return dba_conn.run(sql, (table, username)).get_rows()[0]
     
     @staticmethod
-    def compare_table(src_dba_conn, src_username, dst_dba_conn, dst_username):
-        """ 比较两个不同用户下同名表表结构，输出不一致的表清单，和一段报告 """
+    def compare_table(src_conn, src_username, src_table_name, dst_conn, dst_username, dst_table_name):
+        """ 比较两个表，根据desc方法得到的字符串去比较 """
         src_username = src_username.upper()
         dst_username = dst_username.upper()
-        out = ''
-        table_list = []
-        sql = 'select table_name from dba_tables where owner = ?'
-        srctabs = src_dba_conn.run(sql, (src_username,)).get_rows()
-        for tab in srctabs:
-            src_desc = OracleTools.desc(src_dba_conn, src_username, tab[0])
-            dst_desc = OracleTools.desc(dst_dba_conn, dst_username, tab[0])
-            if src_desc != dst_desc:
-                table_list.append(tab[0])
-                out = out + line_merge_align(src_username + '.' + tab[0] + '\n' + src_desc
-                                           , dst_username + '.' + tab[0] + '\n' + dst_desc
-                                           , True) + '\n\n'
-        return table_list, out
+
+        src_desc = OracleTools.desc(src_conn, src_username, src_table_name)
+        dst_desc = OracleTools.desc(dst_conn, dst_username, dst_table_name)
+        mess = line_merge_align(str_1=src_username + '.' + src_table_name + '\n' + src_desc
+                                       , str_2=dst_username + '.' + dst_table_name + '\n' + dst_desc
+                                       , iscompare=True) + '\n\n'
+        return mess, src_desc == dst_desc
     
-    '''
-    @staticmethod
-    def sync_table(src_ora_conn, src_username, dst_ora_conn, dst_username, table, mode=1):
-        """ 同步表，mode：
-        0：仅输出增量同步表结构的sql
-        1：增量同步表结构
-        2：重建表结构，不包含外键
-        3：重建表，且同步数据，数据量大的问题暂没考虑 """
-        report = ''
-        if mode == 0:
-            sqls = OracleTools.get_ddl(src_ora_conn, src_username, table)
-            for sql in sqls:
-                sql = sql.replace(src_username + '.', dst_username + '.')
-                report = report + sql + ';\n'
-        if mode == 1:
-            sqls = OracleTools.get_ddl(src_ora_conn, src_username, table)
-            for sql in sqls:
-                sql = sql.replace(src_username + '.', dst_username + '.')
-                try:
-                    dst_ora_conn.run(sql)
-                    report = report + 'ok:' + sql + '\n'
-                except:
-                    report = report + 'err:' + sql + '\n'
-        if mode == 2:
-            try:
-                sql = 'drop table ' + dst_username + '.' + table
-                dst_ora_conn.run(sql)
-                report = report + 'ok:' + sql + '\n'
-            except:
-                report = report + 'err:' + sql + '\n'
-            sqls = OracleTools.get_ddl(src_ora_conn, src_username, table)
-            for sql in sqls:
-                sql = sql.replace(src_username + '.', dst_username + '.')
-                dst_ora_conn.run(sql)
-                report = report + 'ok:' + sql + '\n'
-        if mode == 3:
-            try:
-                sql = 'drop table ' + dst_username + '.' + table + ' cascade constraints purge'
-                # dst_ora_conn.run(sql)
-                report = report + 'ok:' + sql + '\n'
-            except:
-                report = report + 'err:' + sql + '\n'
-            sqls = OracleTools.get_ddl(src_ora_conn, src_username, table)
-            for sql in sqls:
-                sql = sql.replace(src_username + '.', dst_username + '.')
-                try:
-                    # dst_ora_conn.run(sql)
-                    report = report + 'ok:' + sql + '\n'
-                except:
-                    report = report + 'err:' + sql + '\n'
-            sql = 'select * from ' + src_username + '.' + table
-            conn = src_ora_conn.dbpool.acquire()
-            cur = conn.cursor()
-            cur.execute(sql)
-            while True:
-                rs = cur.fetchone()
-                if rs is not None:
-                    pa = str(rs)
-                    sql = 'insert into ' + dst_username + '.' + table + ' values' + pa
-                    # dst_ora_conn.run(sql)
-                    print(sql)
-                else:
-                    break
-            cur.close()
-            conn.close()
-        return report
-    '''
-   
     @staticmethod
     def get_sids_by_host(host_conn):
         """ 根据给入的linux系统tools_hjh.SSHConn对象获取这台主机运行的全部SID实例名称 """
-        try:
-            sids = []
-            pros = host_conn.exec_command("source .bash_profile && cd $ORACLE_HOME/dbs && ls -l init*.ora | awk '{print $9}'").split('\n')
-            for pro in pros:
-                sid = pro.replace('init', '').replace('.ora', '')
-                if len(sid) > 0:
-                    sids.append()
-        except:
-            sids = []
-            pros = host_conn.exec_command("ps -ef | grep ora_smon | grep -v grep | awk '{print $8}'").split('\n')
-            for pro in pros:
-                sids.append(pro.replace('ora_smon_', ''))
+        sids = []
+        pros = host_conn.exec_command("ps -ef | grep ora_smon | grep -v grep | awk '{print $8}'").split('\n')
+        for pro in pros:
+            sids.append(pro.replace('ora_smon_', ''))
         return sids
-
+    
     @staticmethod
     def get_obj_size(dba_conn):
         """ 得到对象(table、index、lob)大小 """
@@ -426,204 +469,6 @@ class OracleTools:
         size = size.replace('\n', '').strip()
         return size, rs_list, mess
     
-    @staticmethod   
-    def expdp_job(sid, users=''):
-        date_str = locatdate()
-        if users == '':
-            sh = '''
-                source ~/.bash_profile
-                export ORACLE_SID=''' + sid + '''
-                expdp \\'/ as sysdba\\' \\
-                compression=all \\
-                cluster=n \\
-                parallel=8 \\
-                full=y \\
-                directory=DUMP \\
-                dumpfile=''' + sid + '_' + date_str + '''.dmp
-            '''
-        else:
-            sh = '''
-                source ~/.bash_profile
-                export ORACLE_SID=''' + sid + '''
-                expdp \\'/ as sysdba\\' \\
-                compression=all \\
-                cluster=n \\
-                parallel=8 \\
-                schemas=''' + users + ''' \\
-                directory=DUMP \\
-                dumpfile=''' + sid + '_' + date_str + '''.dmp
-            '''  
-        
-        return remove_leading_space(sh)
-    
-    @staticmethod  
-    def expdp(src_host_conn, src_dba_conn, src_ora_sid, src_ora_user='', dump_tmp_py='/tmp/oracle_dump_py'):
-        if src_ora_user == '':
-            src_ora_user = 'all'
-        date_str = locatdate()
-        dump_filename = src_ora_user + '_' + date_str + '.dmp'
-        log = Log('logs/expdp.log')
-        src_ora_user = ''
-        
-        # 源端创建DUMP_TMP_PY
-        src_host_ip = src_host_conn.host
-        try:
-            src_dba_conn.run('drop directory DUMP_TMP_PY')
-        except Exception as e:
-            pass
-        try:
-            src_dba_conn.run("create directory DUMP_TMP_PY as '" + dump_tmp_py + "'")
-            log.info('源端', src_host_ip, 'DUMP_TMP_PY 创建成功')
-        except Exception as e:
-            log.error('源端', src_host_ip, 'DUMP_TMP_PY 创建失败，程序终止', e)
-            log.error(str(e))
-            return False
-        
-        # 源端导出dmp
-        if src_ora_user != '':
-            sh = '''
-                rm -f ''' + dump_tmp_py + '''/{dump_filename}
-                source ~/.bash_profile
-                export ORACLE_SID={oracle_sid}
-                expdp \\'/ as sysdba \\' directory=DUMP_TMP_PY dumpfile={dump_filename} schemas={src_ora_user} compression=all cluster=n parallel=4
-            '''
-        else:
-            sh = '''
-                rm -f ''' + dump_tmp_py + '''/{dump_filename}
-                source ~/.bash_profile
-                export ORACLE_SID={oracle_sid}
-                expdp \\'/ as sysdba \\' directory=DUMP_TMP_PY dumpfile={dump_filename} full=y compression=all cluster=n parallel=4
-            '''
-        sh = sh.replace('{oracle_sid}', src_ora_sid)
-        sh = sh.replace('{dump_filename}', dump_filename)
-        sh = sh.replace('{src_ora_user}', src_ora_user)
-        mess = src_host_conn.exec_script(sh)
-        if 'successfully completed' in mess:
-            log.info('源端', src_host_ip, 'DMP导出成功')
-        else:
-            log.error('源端', src_host_ip, 'DMP导出失败，程序终止')
-            log.error(mess)
-            return False
-        return True
-
-    @staticmethod     
-    def expdp_scp_impdp(src_host_conn, src_dba_conn, src_ora_sid, src_ora_user, dst_host_conn, dst_dba_conn, dst_ora_sid, dump_tmp_py='/tmp/oracle_dump_py'):
-        """ 
-        0.需要事项创建好表空间, src_ora_user给入''表示full=y
-        1.源端和目标端新建 directory DUMP_TMP_PY /tmp/oracle_dump_py
-        2.源端获取 需要导出用户的表空间名称和大小
-        3.源端执行 expdp 导出数据文件到 DUMP_TMP_PY
-        4.scp文件到目标端 dump_tmp_py
-        5.目标端执行 impdp 导入数据文件
-        """
-        if src_ora_user == '':
-            src_ora_user = 'all'
-        date_str = locatdate()
-        dump_filename = src_ora_user + '_' + date_str + '.dmp'
-        dump_logname = src_ora_user + '_' + date_str + '.log'
-        log = Log('logs/expdp_scp_impdp.log')
-        src_ora_user = ''
-        
-        # 源端创建DUMP_TMP_PY
-        src_host_ip = src_host_conn.host
-        try:
-            src_dba_conn.run('drop directory DUMP_TMP_PY')
-        except Exception as e:
-            pass
-        try:
-            src_dba_conn.run("create directory DUMP_TMP_PY as '" + dump_tmp_py + "'")
-            log.info('源端', src_host_ip, 'DUMP_TMP_PY 创建成功')
-        except Exception as e:
-            log.error('源端', src_host_ip, 'DUMP_TMP_PY 创建失败，程序终止', e)
-            log.error(str(e))
-            return False
-        
-        # 目标端创建DUMP_TMP_PY
-        dst_host_ip = dst_host_conn.host
-        try:
-            dst_dba_conn.run('drop directory DUMP_TMP_PY')
-        except Exception as e:
-            pass
-        try:
-            dst_dba_conn.run("create directory DUMP_TMP_PY as '" + dump_tmp_py + "'")
-            log.info('目标端', dst_host_ip, 'DUMP_TMP_PY 创建成功')
-        except Exception as e:
-            log.error('目标端', dst_host_ip, 'DUMP_TMP_PY 创建失败，程序终止', e)
-            log.error(str(e))
-            return False
-        
-        # 源端导出dmp
-        if src_ora_user != '':
-            sh = '''
-                rm -f ''' + dump_tmp_py + '''/{dump_filename}
-                source ~/.bash_profile
-                export ORACLE_SID={oracle_sid}
-                expdp \\'/ as sysdba \\' directory=DUMP_TMP_PY dumpfile={dump_filename} schemas={src_ora_user} compression=all cluster=n parallel=4
-            '''
-        else:
-            sh = '''
-                rm -f ''' + dump_tmp_py + '''/{dump_filename}
-                source ~/.bash_profile
-                export ORACLE_SID={oracle_sid}
-                expdp \\'/ as sysdba \\' directory=DUMP_TMP_PY dumpfile={dump_filename} full=y compression=all cluster=n parallel=4
-            '''
-        sh = sh.replace('{oracle_sid}', src_ora_sid)
-        sh = sh.replace('{dump_filename}', dump_filename)
-        sh = sh.replace('{src_ora_user}', src_ora_user)
-        mess = src_host_conn.exec_script(sh)
-        if 'successfully completed' in mess:
-            log.info('源端', src_host_ip, 'DMP导出成功')
-        else:
-            log.error('源端', src_host_ip, 'DMP导出失败，程序终止')
-            log.error(mess)
-            return False
-
-        # scp到目标端
-        sh = '''
-            source ~/.bash_profile
-            expect -c "
-            spawn scp -P {dst_host_port} -r ''' + dump_tmp_py + '''/{dump_filename} {dst_host_user}@{dst_host_ip}:''' + dump_tmp_py + '''/
-            expect {
-                \\"*assword\\" {set timeout 30; send \\"{dst_host_password}\\r\\";}
-                \\"yes/no\\" {send \\"yes\\r\\"; exp_continue;}
-            }
-            expect eof"
-        '''
-        dst_host_port = dst_host_conn.port
-        dst_host_user = dst_host_conn.username
-        dst_host_password = dst_host_conn.password
-        sh = sh.replace('{dst_host_port}', dst_host_port)
-        sh = sh.replace('{dump_filename}', dump_filename)
-        sh = sh.replace('{dst_host_user}', dst_host_user)
-        sh = sh.replace('{dst_host_ip}', dst_host_ip)
-        sh = sh.replace('{dst_host_password}', dst_host_password)
-        mess = src_host_conn.exec_script(sh)
-        if '100%' in mess:
-            log.info('源端', src_host_ip, 'SCP成功')
-        else:
-            log.error('源端', src_host_ip, 'SCP失败，程序终止')
-            log.error(mess)
-            return False
-        
-        # 目标端导入dmp
-        sh = '''
-            source ~/.bash_profile
-            export ORACLE_SID={oracle_sid}
-            impdp \\'/ as sysdba \\' directory=DUMP_TMP_PY dumpfile={dump_filename} logfile={dump_logname} full=y transform=segment_attributes:n table_exists_action=replace
-        '''
-        sh = sh.replace('{oracle_sid}', dst_ora_sid)
-        sh = sh.replace('{src_ora_user}', src_ora_user)
-        sh = sh.replace('{dump_filename}', dump_filename)
-        sh = sh.replace('{dump_logname}', dump_logname)
-        mess = dst_host_conn.exec_script(sh)
-        if 'completed' in mess:
-            log.info('目标端', dst_host_ip, 'DMP导入结束')
-        else:
-            log.error('目标端', dst_host_ip, 'DMP导入失败，程序终止')
-            log.error(mess)
-            return False
-        return True
-        
     @staticmethod
     def analysis_tns(host_conn):
         """ 解析Oracle tnsnames.ora文件 """
@@ -668,17 +513,22 @@ class OracleTools:
         """ 进入主机全部找到的ggsci，执行info all 返回结果 """
         query_time = locattime()
         host = host_conn.host
+        username = host_conn.username
         
         # 进程状态 
         # 查询时间 ogg所在主机HOST ggsci所在路径 进程类型 进程状态 进程名称 lag_at_chkpt time_since_chkpt
         ogg_status = QueryResults()
+        ogg_status.get_rows().clear()
         ogg_status.set_cols(('query_time', 'host', 'ggsci_path', 'type', 'status', 'name', 'lag_at_chkpt', 'time_since_chkpt'))
         
         # 解析进程状态 
         cmd = 'locate *ggsci'
         paths = host_conn.exec_command(cmd)
         for path in paths.split('\n'):
-            cmd = 'source ~/.bash_profile;echo "info all" | ' + path
+            if username == 'oracle':
+                cmd = 'source ~/.bash_profile;echo "info all" | ' + path
+            else:
+                cmd = '''su - oracle -c 'source ~/.bash_profile;echo "info all" | ''' + path + '\''
             mess = host_conn.exec_command(cmd)
             for line in mess.split('\n'):
                 if line.startswith('MANAGER'):
