@@ -39,14 +39,6 @@ class SSHConn:
             mess = mess + shell.recv(1024).decode('UTF-8', errors='ignore')
         shell.close()
         return mess
-    
-    def console(self, script):
-        """ 执行shell脚本，返回交互式shell内容 """
-        script = remove_leading_space(script)
-        self.shell.send(script + '\n')
-        # time.sleep(0.5)
-        mess = self.shell.recv(1024).decode('UTF-8', errors='ignore')
-        return mess
         
     def _download(self, src, dst):
         if not os.path.exists(os.path.dirname(dst)):
