@@ -163,7 +163,7 @@ def line_merge_align(str_1, str_2, iscompare=False):
         except:
             str2 = ''
         if iscompare:
-            if line_merge_align(str1.strip()) != line_merge_align(str2.strip()):
+            if merge_spaces(str1.strip()) != merge_spaces(str2.strip()):
                 str_ = str_ + str1 + ' ' * (strlennum - len(str1) + 1) + str2 + '\t*\n'
             else:
                 str_ = str_ + str1 + ' ' * (strlennum - len(str1) + 1) + str2 + '\n'
